#include <SPI.h>
#include <TFT.h> 

#define PIN_TFT_CS 7
#define PIN_DC 9
#define PIN_RST 8

// PINS used
const int buzzer = 3;
const int redPin = 10;
const int greenPin = 5;
const int bluePin = 6;
const int SW_pin = 2; // digital pin connected to switch output
const int X_pin = 0; // analog pin connected to X output
const int Y_pin = 1; // analog pin connected to Y output

const int width = 160;
const int height = 128;

// main menu variables
int mainMenuCursor = 1, lastMainMenuCursor = 1;
bool isInMenu = 1;
int gameType = 0;

// game related variables
int posx = 0, posy = 0; 
int cursorX, cursorY;  // position of the cursor on the TTT table
bool playerTurn = 1;   // 1 -> 'X' turn ; 0 -> 'O' turn
int matr[3][3] = {0};  // 0 -> empty space; 1 -> 'X'; 2 -> 'O'
unsigned long lastMovedCursor = 0;
int lastButtonState = HIGH;

// sound related variables
int melody[] = {
  262, 196, 196};
int noteDurations[] = {
  12, 4, 4};
const int numberOfBadNotes = 3;
TFT TFTscreen = TFT(PIN_TFT_CS, PIN_DC, PIN_RST);
void setColor(int redValue, int greenValue, int blueValue);

void drawFirstPage() {
   TFTscreen.background(0, 0, 0);
   TFTscreen.drawRect(width / 6, height / 6, 2 * width / 3, height / 3, 0x00FF00);
   TFTscreen.textSize(2);
   TFTscreen.stroke(255, 255, 255);
   TFTscreen.text("P v P", width / 3, height / 4);

   TFTscreen.drawRect(width / 6, height / 2 + height / 12, 2 * width / 3, height / 3, 0x00FF00);
   TFTscreen.text("P v Com", width / 4, height - height / 4);

}
void drawBackground() {
   TFTscreen.background(0, 0, 0);
   TFTscreen.stroke(255, 255, 255);
   TFTscreen.line(width / 3, 1, width / 3, height - 1);
   TFTscreen.line(2 * width / 3, 1, 2 * width / 3, height - 1); 
   TFTscreen.line(1, height / 3, width - 1, height / 3);
   TFTscreen.line(1, 2 * height / 3, width - 1, 2 * height / 3);
}
int resultOfGame() {
  // 0 -> not finished; 1 -> X win; 2 -> O win; 3 -> draw

  // check both diagonals
  if (matr[0][0] == matr[1][1] && matr[1][1] == matr[2][2] && matr[0][0] != 0)
      return matr[0][0];
  if (matr[0][2] == matr[1][1] && matr[1][1] == matr[2][0] && matr[0][2] != 0) 
      return matr[0][2];
  
  for (int i = 0; i < 3; ++i) {
       // check lines and cols
      if (matr[0][i] != 0 && matr[0][i] == matr[1][i] && matr[0][i] == matr[2][i]) {
          return matr[0][i];
      }
      if (matr[i][0] != 0 && matr[i][0] == matr[i][1] && matr[i][1] == matr[i][2]) {
          return matr[i][0];
      }
  }
  for (int i = 0; i < 3; ++i) {
      for (int j = 0; j < 3; ++j) {
           if (matr[i][j] == 0)
              return 0;
      }
  }
  return 3;
}
void setupGame() {
  playerTurn = 1;
  cursorX = 0;
  cursorY = 0;
  for (int i = 0; i < 3; ++i) {
    for (int j = 0; j < 3; ++j) {
       matr[i][j] = 0;
    }
  }
}
void playBadChoice() {
   for (int thisNote = 0; thisNote < numberOfBadNotes; thisNote++) {
              int noteDuration = 1000/noteDurations[thisNote];
              tone(buzzer, melody[thisNote],noteDuration);
              int pauseBetweenNotes = noteDuration * 1.30;
              delay(pauseBetweenNotes);
              noTone(buzzer);
    }
}
void makeMove() {
   if (playerTurn) {
      if (matr[cursorX][cursorY] != 0) {
          playBadChoice();
          return;
      }
      int x1 = cursorX * width / 3 + width / 6;
      int y1 = cursorY * height / 3 + height / 6;
      int x2 = width / 3 - width / 6;
      int y2 = height / 3 - height / 6;
      TFTscreen.drawCircle(x1, y1, 5, 0x000000);
      TFTscreen.drawLine(x1 - x2, y1 - y2, x1 + x2, y1 + y2, 0xFFFFFF);
      TFTscreen.drawLine(x1 + x2, y1 - y2, x1 - x2, y1 + y2, 0xFFFFFF);
      playerTurn = 0;
      setColor(0, 0, 40); // blue Color
      matr[cursorX][cursorY] = 1;
   } else {
     if (matr[cursorX][cursorY] != 0) {
          playBadChoice();
          return;
      }
     int x1 = cursorX * width / 3 + width / 6;
     int y1 = cursorY * height / 3 + height / 6;
     TFTscreen.drawCircle(x1, y1, 5, 0x000000);
     TFTscreen.drawCircle(x1, y1, 10, 0xFFFFFF);
     playerTurn = 1;
     setColor(40, 0 , 0); // red color
     matr[cursorX][cursorY] = 2;
   }
}
void makeMoveVsOpponent() {
  // your move
  if (matr[cursorX][cursorY] != 0) {
          playBadChoice();
          return;
  }
  int x1 = cursorX * width / 3 + width / 6;
  int y1 = cursorY * height / 3 + height / 6;
  int x2 = width / 3 - width / 6;
  int y2 = height / 3 - height / 6;
  TFTscreen.drawCircle(x1, y1, 5, 0x000000);
  TFTscreen.drawLine(x1 - x2, y1 - y2, x1 + x2, y1 + y2, 0xFFFFFF);
  TFTscreen.drawLine(x1 + x2, y1 - y2, x1 - x2, y1 + y2, 0xFFFFFF);
  matr[cursorX][cursorY] = 1;
  
  int checkResult = resultOfGame();
  if (checkResult != 0)
      return;
  // opponent's move
  if (matr[1][1] == 0) {
     int x1 = width / 3 + width / 6;
     int y1 = height / 3 + height / 6;
     TFTscreen.drawCircle(x1, y1, 10, 0xFFFFFF);
     matr[1][1] = 2;
     return;
  }
  int movesArray[9], countMoves = 0;
  for (int i = 0; i < 3; ++i) {
      for (int j = 0; j < 3; ++j) {
          if (matr[i][j] == 0) {
              movesArray[countMoves] = i * 3 + j;
              countMoves++;
          }              
      }
  }
  if (countMoves <= 0) {
      return;
  }
  int random = rand() % countMoves, i, j;
  i = movesArray[random] / 3;
  j = movesArray[random] % 3;
  x1 = i * width / 3 + width / 6;
  y1 = j * height / 3 + height / 6;
  TFTscreen.drawCircle(x1, y1, 10, 0xFFFFFF);
  matr[i][j] = 2;
}
void moveCursor() {
   unsigned long cursorTime = millis();
   if (cursorTime - lastMovedCursor < 500) {
       return; 
   }
   TFTscreen.drawCircle(cursorX * width / 3 + width / 6, cursorY * height / 3 + height / 6, 5, 0x000000);
   int joyval = analogRead(X_pin);
   joyval = map(joyval, 0, 750, -1, 2);
   
   cursorX += joyval;
   if (cursorX < 0) {
       cursorX = 0;
   }
   if (cursorX > 2) {
      cursorX = 2;
   }
   joyval = analogRead(Y_pin);
   joyval = map(joyval, 0, 750, -1, 2);
   cursorY += joyval;
   if (cursorY < 0) {
       cursorY = 0;
   }
   if (cursorY > 2) {
       cursorY = 2;
   }
   lastMovedCursor = cursorTime;
}
void moveMenuCursor() {
   //TFTscreen.drawCircle(width / 12, mainMenuCursor * height / 3 + (mainMenuCursor - 1) * height / 12, 5, 0x000000);
   int joyval = analogRead(Y_pin);
   joyval = map(joyval, 0, 750, -1, 2);
   mainMenuCursor += joyval;
   if (mainMenuCursor < 1) {
       mainMenuCursor = 1;
   }
   if (mainMenuCursor > 2) {
       mainMenuCursor = 2;
   }
}
void setup() {
  // initialize default serial connection to send debug information
  Serial.begin(9600);
  while (!Serial) {
    // wait until default serial connection is fully set up
  }
  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT);

  //The following two lines replace "TFTscreen.begin();" => avoids that red and blue (?) are swapped/interchanged
  TFTscreen.initR(INITR_BLACKTAB);
  TFTscreen.setRotation(1);
  pinMode(SW_pin, INPUT);
  digitalWrite(SW_pin, HIGH); 
  //drawBackground();
  drawFirstPage();
}
void endGameText(int result) {
    TFTscreen.background(0, 0, 0);
    if (result == 1) {
      TFTscreen.stroke(255, 0, 0);
      for (int i = 0; i < 5; ++i) {
        TFTscreen.background(0, 0, 0);
        setColor(5, 0, 0);
        TFTscreen.setTextSize(3);
        TFTscreen.text("X WON!", width / 4, height / 2);
        delay(500);
        TFTscreen.background(0, 0, 0);
        setColor(70, 0 ,0);
        TFTscreen.setTextSize(4);
        TFTscreen.text("X WON!", width / 8, height / 2);
        delay(500);
      }
      //delay(5000);
      
  }
  if (result == 2) {
      TFTscreen.stroke(0, 0, 255);
      for (int i = 0; i < 5; ++i) {
        TFTscreen.background(0, 0, 0);  
        setColor(0, 0, 10);
        TFTscreen.setTextSize(3);
        TFTscreen.text("O WON!", width / 4, height / 2);
        delay(500);
        TFTscreen.background(0, 0, 0);
        setColor(0, 0, 90);
        TFTscreen.setTextSize(4);
        TFTscreen.text("O WON!", width / 8, height / 2);
        delay(500);
      }
      // delay(5000);
  }
  if (result == 3) {
      TFTscreen.stroke(255, 255, 0);
      for (int i = 0; i < 5; ++i) {
        TFTscreen.background(0, 0, 0);  
        setColor(10, 10, 0);
        TFTscreen.setTextSize(3);
        TFTscreen.text("DRAW!", width / 4, height / 2);
        delay(500);
        TFTscreen.background(0, 0, 0);
        setColor(90, 90, 0);
        TFTscreen.setTextSize(4);
        TFTscreen.text("DRAW!", width / 8, height / 2);
        delay(500);
      }
      //delay(5000);
  }
  gameType = 0;
  isInMenu = 1;
  drawFirstPage();
}
void PvPGame() {
    int result = resultOfGame();
    setColor(40, 0, 0);
    while (result == 0) {
    moveCursor();
    TFTscreen.drawCircle(cursorX * width / 3 + width / 6, cursorY * height / 3 + height / 6, 5, 0xFFFF00);
    
    int buttonState = digitalRead(SW_pin);
    if (buttonState == HIGH && lastButtonState == LOW) {
        makeMove();
    }
    lastButtonState = buttonState;
    result = resultOfGame();
  }
  endGameText(result);
}
void PvComGame() {
  int result = resultOfGame();
    while (result == 0) {
    moveCursor();
    TFTscreen.drawCircle(cursorX * width / 3 + width / 6, cursorY * height / 3 + height / 6, 5, 0xFFFF00);
    
    int buttonState = digitalRead(SW_pin);
    if (buttonState == HIGH && lastButtonState == LOW) {
        makeMoveVsOpponent();
    }
    lastButtonState = buttonState;
    result = resultOfGame();
  }
  endGameText(result);
}
void loop() {

  // SW_pin == 1 => buton neapasat; SW_pin = 0 => buton apasat;
  //int result = resultOfGame();
  setColor(0, 0, 0);
  TFTscreen.drawCircle(width / 12, mainMenuCursor * height / 3 + (mainMenuCursor - 1) * height / 12, 5, 0x00FF00);
  if (isInMenu) {
      moveMenuCursor();
      if (lastMainMenuCursor != mainMenuCursor) {
          TFTscreen.drawCircle(width / 12, lastMainMenuCursor * height / 3 + (lastMainMenuCursor - 1) * height / 12, 5, 0x000000);
          TFTscreen.drawCircle(width / 12, mainMenuCursor * height / 3 + (mainMenuCursor - 1) * height / 12, 5, 0x00FF00);
      }
      if (digitalRead(SW_pin) == LOW) {
          isInMenu = 0;
          gameType = mainMenuCursor;
          setupGame();
      }
      lastMainMenuCursor = mainMenuCursor;
  }
  if (gameType == 1) {
      drawBackground();
      PvPGame();
  }
  if (gameType == 2) {
      drawBackground();
      delay(500);
      PvComGame();
  }
  
}
void setColor(int redValue, int greenValue, int blueValue) {
  analogWrite(redPin, redValue);
  analogWrite(greenPin, greenValue);
  analogWrite(bluePin, blueValue);
}